#include <iostream>
#include "stacktrace_Win32.hpp"

#include <vector>
#include <iterator>
#include <iostream>

// See main() for description
//#define REDIRECT_DLL

void f1()
{
  std::cout << "\nWalking the stack, Direct implementation:\n\n";

  std::vector<void *> stack;
  boost::walk_stack(std::back_inserter(stack), boost::stack_Win32_use_direct());

  std::vector<std::string> symbols;
  boost::stack_symbols(stack.begin(), stack.end(), std::back_inserter(symbols));

  for (std::vector<std::string>::const_iterator i = symbols.begin(); i != symbols.end(); ++i)
    std::cout << *i << "\n";
}

void f2()
{
  std::cout << "\nWalking the stack, Exception implementation:\n\n";

  std::vector<void *> stack;
  boost::walk_stack(std::back_inserter(stack), boost::stack_Win32_use_exception());

  std::vector<std::string> symbols;
  boost::stack_symbols(stack.begin(), stack.end(), std::back_inserter(symbols));

  for (std::vector<std::string>::const_iterator i = symbols.begin(); i != symbols.end(); ++i)
    std::cout << *i << "\n";
}

void f3()
{
  std::cout << "\nWalking the stack, Thread implementation:\n\n";

  std::vector<void *> stack;
  boost::walk_stack(std::back_inserter(stack), boost::stack_Win32_use_thread());

  std::vector<std::string> symbols;
  boost::stack_symbols(stack.begin(), stack.end(), std::back_inserter(symbols));

  for (std::vector<std::string>::const_iterator i = symbols.begin(); i != symbols.end(); ++i)
    std::cout << *i << "\n";
}

void f(void (*func)()) { func(); }

int main()
{
#ifdef REDIRECT_DLL
  // The dbghelp.dll redistributable is in "dll\dbghelp.dll"
  // In a real application, you would only specify this directory if
  //  running on 95/98/Me/NT.
  // Or better yet, have your installer put "dbghelp.dll" in you app's exe
  //  directory only if installing on 95/98/Me/NT -- then you won't have
  //  to redirect it at all.  But we DON'T want the installer to put it
  //  there on 2000/XP/+, because there would be a dbghelp.dll in the
  //  System already with equivalent or better functionality...
  // Toolhelp (kernel32.dll) and PSAPI (psapi.dll) can also be redirected.
  //  But I can't think of a reason why you would want to...
  // Defining REDIRECT_DLL on an XP or .NET Server box has the net effect of
  //  forcing Boost.StackWalk to act like this is a 2000 box.
  boost::stack_Win32_set_dbghelp_dll_location("dll\\dbghelp.dll");
#endif

  f(f1);
  f(f2);

  // Let's say that we've unloaded and/or loaded some DLLs, and we want to
  //  make Boost.StackWalk aware of that fact.  We would call:
  boost::stack_Win32_refresh_module_list();

  f(f3);
  return 0;
}